import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
  line,
  curveBasis,
  nest,
  schemeCategory10,
  descending
  
} from 'd3';
import 'dayjs';

import { colorLegend } from './colorLegend';
const svg = select('svg');

// Add height and width variables
const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  const title = 'Temperature Display';
  
  const xValue = d => d.date; // Parse the date string from csv data
  const xAxisLabel = 'Months';
  
  const yValue = d => d.temperature;// Parse the temperature string from csv data
  const circleRadius = 6;
  const yAxisLabel = 'Temperature';
  
  const colorValue = d => d.district;
  
  // Locate the chart
  const margin = { top: 60, right: 160, bottom: 88, left: 105 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  // Scale the x-axis length
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  
  // Scale the y-axis length
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  const colorScale = scaleOrdinal(schemeCategory10);
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  // Takes care of x-axis attributes
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  // Takes care of y-axis attributes
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  // Add attributes to y_axis
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -60)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  xAxisG.select('.domain').remove();
  
  // Add attributes to y_axis
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 80)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  // Generate the line based on x and y scale values
  const lineGenerator = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale(yValue(d)))
    .curve(curveBasis);
  
  // Give a small distance to the end of the graph
  const lastYValue = d =>
    yValue(d.values[d.values.length - 1]);
  
  // Distribute colors based from data
  const nested = nest()
    .key(colorValue)
    .entries(data)
    .sort((a, b) =>
      descending(lastYValue(a), lastYValue(b))
    );
  
  colorScale.domain(nested.map(d => d.key));
  
  // Control the graph line
  g.selectAll('.line-path').data(nested)
    .enter().append('path')
      .attr('class', 'line-path')
      .attr('d', d => lineGenerator(d.values))
      .attr('stroke', d => colorScale(d.key));
  
  // Control the title
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);
};

// Read from the source and parse the values
csv('https://raw.githubusercontent.com/teohangxanh/5320/main/ICE-7/Fake%20temperature.csv')
  .then(data => {
    data.forEach(d => {
      d.temperature = +d.temperature;
      d.date = dayjs(d.date, "DD/MM/YYYY");
    });
  

    render(data);
  });

